"use strict"

var request = require('supertest')
  , cookieParser 	= require('cookie-parser')
  , bodyParser = require("body-parser")
  , express = require('express')
  , should  = require('chai').should
  , expect  = require('chai').expect
  , vericode  = require('../')
  , HuyiSmsHandler = require('sms-gateway-huyi')
  , verificationProvider = require('../libs/Sqlite3VeriProvider');

describe('Verfication Code Module Mock', function(){

  process.env.NODE_ENV = 'local-unit-test';
  var code;
  var vericode_token;
  var identity = "lei.he@aivics.com";

  var app = express();
  app.use(cookieParser());
  app.use(bodyParser());
  app.use(vericode({
    whitelist: [ /\/.*/ ]
    ,verificationProvider: verificationProvider
    ,expires: 5 * 60 * 1000
    ,smsTemplate: "您抢到的Fun抢福袋价值888元扮靓大福袋的代码为${veri_code}，请在2月28日前前往C&A官网www.canda.cn使用，具体使用规则请参见活动说明"
    ,smsHandler: HuyiSmsHandler({
      gateway: 'http://121.199.16.178/webservice/sms.php?method=Submit'
      ,account: 'cf_obizsoft'
      ,password: 'a123456'
    })
    ,email: {
      port: 465,
      host: 'smtp.mxhichina.com',
      user: 'service@aivics.net',
      pass: '001@Helei',
      secure: true,
      debug: true,
      from: 'service@aivics.net'
    }
  }));


  it('请求应该被拒绝', function(done){
    app.use(function(req, res, next){
      return res.status(200).end();
    });

    request(app)
      .get('/identity/administrators/activate')
      .expect(403, done);
  });

  it('should send verification code', function(done){
    var data = { identity: identity , veri_type: 'email', target_url: "/" };

    request(app)
      .post('/vericode/code')
      .send( data )
      .expect(201)
      .expect(function(res){

        code = res.body.veri_code;

        if( res.body.success !== true ) throw new Error('invalid_response');
        if( process.env.NODE_ENV === 'local-unit-test' && !res.body.veri_code ) throw new Error('invalid_code_returned');
      })
      .end(done);
  });

  it('should return token', function(done){

    console.log(identity + " & " + code);

    var data = { identity: identity, veri_code: code };
    request(app)
      .post('/vericode/validate')
      .send( data )
      .expect(201)
      .end(function(err, res){
        if (err) return done(err);
        vericode_token = res.body.vericode_token;
        done();
      });
  });


  it('should success pass token verification', function(done){
    app.use(function(req, res, next){
      return res.status(200).end();
    });

    request(app)
      .get('/')
      .send({ vericode_token: vericode_token })
      .expect(200, done);
  });

  it('should failed to pass token verification with incorrect targetUrl', function(done){
    app.use("/test", function(req, res, next){
      return res.status(200).end();
    });

    request(app)
      .get('/test')
      .send({ vericode_token: vericode_token })
      .expect(403, {
        success: false,
        errCode: 403,
        errMsg: "access_denied_because_of_url_not_match"
      })
      .end(done);
  });
});


describe('Verfication Code Module code expires feature', function(){

  process.env.NODE_ENV = 'local-unit-test';
  var code;
  var identity = "lei.he@aivics.com";

  var app = express();
  app.use(cookieParser());
  app.use(bodyParser());
  app.use(vericode({
    whitelist: [ /\/.*/ ]
    ,verificationProvider: verificationProvider
    ,expires: 0 * 1000
    ,smsTemplate: "您抢到的Fun抢福袋价值888元扮靓大福袋的代码为${veri_code}，请在2月28日前前往C&A官网www.canda.cn使用，具体使用规则请参见活动说明"
    ,smsHandler: HuyiSmsHandler({
      gateway: 'http://121.199.16.178/webservice/sms.php?method=Submit'
      ,account: 'cf_obizsoft'
      ,password: 'a123456'
    })
    ,email: {
      port: 465,
      host: 'smtp.mxhichina.com',
      user: 'service@aivics.net',
      pass: '001@Helei',
      secure: true,
      debug: true,
      from: 'service@aivics.net'
    }
  }));


  it('should send verification code', function(done){
    var data = { identity: identity , veri_type: 'email', target_url: "/" };

    request(app)
      .post('/vericode/code')
      .send( data )
      .expect(201)
      .expect(function(res){
        console.log(res.body);
        code = res.body.veri_code;

        if( res.body.success !== true ) throw new Error('invalid_response');
        if( process.env.NODE_ENV === 'local-unit-test' && !res.body.veri_code ) throw new Error('invalid_code_returned');
      })
      .end(done);
  });

  it('should return code_expired error', function(done){
    var data = { identity: identity, veri_code: code };
    request(app)
      .post('/vericode/validate')
      .send( data )
      .expect(403, { success: false, errCode: 403, errMsg: "code_expired" })
      .end(function(err, res){
        console.log(res.body);
        if (err) return done(err);
        done();
      });
  });
});

//
// describe('Expect Code Expires', function(){
//   it('should return 500', function(done){
//     var app = express();
//     app.use(cookieParser());
//     app.use(bodyParser());
//     app.use(vericode({
//       whitelist: [ /\/.*/ ]
//       ,verificationProvider: verificationProvider
//       ,expires: 0 * 1000
//       ,smsTemplate: "您抢到的Fun抢福袋价值888元扮靓大福袋的代码为${veri_code}，请在2月28日前前往C&A官网www.canda.cn使用，具体使用规则请参见活动说明"
//       ,smsHandler: HuyiSmsHandler({
//         gateway: 'http://121.199.16.178/webservice/sms.php?method=Submit'
//         ,username: 'cf_obizsoft'
//         ,password: 'a123456'
//       })
//     }));
//
//     var data = { identity: 'XXX', veri_type: 'mobile' };
//     process.env.NODE_ENV = 'local-unit-test';
//     request(app)
//       .post('/vericode/code')
//       .send( data )
//       .end(function(err, res){
//         code = res.body.veri_code;
//         var data = { identity: '13764211365', veri_code: code };
//         request(app)
//           .post('/vericode/validate')
//           .send( data )
//           .expect(500, done);
//       });
//   });
// });
//
//
// describe('Success Pass Through Code Validation', function(){
//   it('should return 200', function(done){
//     var app = express();
//     app.use(cookieParser());
//     app.use(bodyParser());
//     app.use(vericode({
//       whitelist: [ /\/.*/ ]
//       ,verificationProvider: verificationProvider
//       ,expires: 10 * 1000
//       ,smsTemplate: "您抢到的Fun抢福袋价值888元扮靓大福袋的代码为${veri_code}，请在2月28日前前往C&A官网www.canda.cn使用，具体使用规则请参见活动说明"
//       ,smsHandler: HuyiSmsHandler({
//         gateway: 'http://121.199.16.178/webservice/sms.php?method=Submit'
//         ,username: 'xxx'
//         ,password: 'xxx'
//       })
//     }));
//     app.use(function(req, res, next){
//       return res.status(200).end();
//     });
//
//     var data = { identity: '13764211365', veri_type: 'mobile' };
//     process.env.NODE_ENV = 'local-unit-test';
//     request(app)
//       .post('/vericode/code')
//       .send( data )
//       .end(function(err, res){
//         code = res.body.veri_code;
//         var data = { identity: '13764211365', veri_code: code };
//         request(app)
//           .post('/vericode/validate')
//           .send( data )
//           .end(function(err, res){
//             var token = res.body.veri_token;
//
//
//             request(app)
//               .get('/')
//               .send({ veri_token: token })
//               .expect(200, done);
//           });
//       });
//   });
// });
